#!/bin/sh
#
# Script to update kernel and/or rootfs from USB stick


# Check if required files are present 

if [ ! -f "./fw_printenv" ] ; 
then
	echo Missing file : fw_printenv
	exit 0
fi

if [ ! -f "./fw_setenv" ] ; 
then
	echo Missing file : fw_setenv
	exit 0
fi

if [ ! -f "./fw_env.config" ] ; 
then
	echo Missing file : fw_printenv
	exit 0
fi

if [ ! -f "./kernel" ] ; 
then
	echo Missing file : kernel
	exit 0
fi

if [ ! -f "./rootfs.img" ] ; 
then
	echo Missing file : rootfs.img
	exit 0
fi



# first find out which mtd partition us used by the rootfs

# Find names of root mtd partitions


mtd_root1_fs_p=`grep \"root1\" /proc/mtd | sed -e 's/\://g'`

if [ "mtd_root1_fs_p" == "" ] ; then
   echo "First mtd rootfs partition not found."
   echo "Update failed !."
   exit 0
else
   mtd_root1_name=`echo $mtd_root1_fs_p | cut -d ' ' -f 1`
fi

echo "Name of root1 partition is \"$mtd_root1_name\" "

# Find names of root mtd partitions
mtd_root2_fs_p=`grep \"root2\" /proc/mtd | sed -e 's/\://g'`

if [ "mtd_root2_fs_p" == "" ] ; then
   echo "Second mtd rootfs partition not found."
   echo "Update failed !."
   exit 0
else
   mtd_root2_name=`echo $mtd_root2_fs_p | cut -d ' ' -f 1`
fi

echo "Name of root2 partition is \"$mtd_root2_name\" "

# Check kernel command line to find out which partition we booted from

#kernel_boot_part=`grep $mtd_root1_name /proc/cmdline | sed -e 's/\://g'`

#echo "kernel_boot_part is $kernel_boot_part "


#if [ "$kernel_boot_part" == "" ] ; then
#	echo "Kernel did not boot from $mtd_root1_name "
#else
#	echo "Kernel booted from $mtd_root1_name "
#fi

#kernel_boot_part=`grep $mtd_root2_name /proc/cmdline | sed -e 's/\://g'`

#if [ "$kernel_boot_part" == "" ] ; then
#	echo "Kernel did not boot from $mtd_root2_name"
#	echo "No MTD rootfs partitions found in kernel commandline. Update process cannot continue. Maybe you are mounted over NFS ?"
#	exit 0
#else
#  echo "Kernel booted from $mtd_root1_name "
#fi

echo "Welcome to Pixsan/Skardin/Eurosat USB update tool"
echo "Press Enter to begin the update process or CTRL C to abort"

bootargs=`./fw_printenv | grep "ubi.mtd=4"`

echo "bootargs = $bootargs "

# Get current kernel command line !!!! CHANGE BEFORE RUNNING ON TARGET  !!!!

current_boot_args=`cat /proc/cmdline | grep console`


# Function to write kernel and/or rootfs to rootfs 1 
update_flash_root1() {
  	echo writing image to rootfs1
	echo erasing flash 
  	flash_erase /dev/mtd4 0 0
	echo writing new kernel image
	nandwrite -p /dev/mtd4 kernel
	echo writing image to rootfs1
	flash_erase /dev/mtd5 0 0
  	ubiformat /dev/mtd5 -f rootfs.img
	echo Need to update bootargs to boot from mtd.5
	echo Current bootargs is $current_boot_args
	echo $current_boot_args > boot_params1
	sed -e s/"ubi.mtd=8"/"ubi.mtd=5"/g boot_params1 > text1.out
	sed -i 's/bigphysarea=[0-9][0-9]*/bigphysarea=4000/g' text1.out
	new_boot_args=`cat text1.out`
	echo New args are $new_boot_args
	./fw_setenv bootargs $new_boot_args
	./fw_setenv bootcmd "nand read 0x80000000 0 0x400000 ; bootm 0x80000000"
	sync
	echo "The update was successful - rebooting"
	reboot

}

# Function to write kernel and/or rootfs to rootfs 2
update_flash_root2() {
	echo Writing image to rootfs2
	echo erasing flash 
  	flash_erase /dev/mtd4 0 0
	echo writing new kernel image
	nandwrite -p /dev/mtd4 kernel
	echo writing image to rootfs2
	flash_erase /dev/mtd8 0 0  	
	ubiformat /dev/mtd8 -f rootfs.img
# Set kernel parameters for NEW kernel	
  	echo Need to update bootargs to boot from mtd.8
	echo Current bootargs is $current_boot_args
	echo $current_boot_args > boot_params2
	sed -e s/"ubi.mtd=5"/"ubi.mtd=8"/g boot_params2 > text2.out
	sed -i 's/bigphysarea=[0-9][0-9]*/bigphysarea=4000/g' text2.out
	new_boot_args=`cat text2.out`
#copy new parameter to U - boot env 
	echo New args are $new_boot_args
	./fw_setenv bootargs $new_boot_args
	./fw_setenv bootcmd "nand read 0x80000000 0 0x400000 ; bootm 0x80000000"
	sync
	echo "The update was successful - rebooting"
  reboot
}

# Function to write kernel and/or rootfs to rootfs and force boot params
update_flash_root3() {
  echo Ready to write to rootfs2 Press enter to start
	read
	echo Writing image to rootfs2
	echo erasing flash 
  	flash_erase /dev/mtd4 0 0
	echo writing new kernel image
	nandwrite -p /dev/mtd4 kernel
	echo writing image to rootfs2
	flash_erase /dev/mtd5 0 0  	
	ubiformat /dev/mtd5 -f rootfs.img -s 512
  	`echo "console=ttyAS0,115200 ubi.mtd=5 root=ubi0:rootfs rootfstype=ubifs nwhwconf=device:eth0,hwaddr:00:FA:E0:FA:E0:01 ip=dhcp bigphysarea=4000" > text3.out`
  	new_boot_args=`cat text3.out`
	echo New args are $new_boot_args
	./fw_setenv bootargs $new_boot_args
	./fw_setenv bootcmd "nand read 0x80000000 0 0x400000 ; bootm 0x80000000"
	sync
	echo "The update was successful - Please restart your box"

}

# Check kernel params for which part we booted from 
# Try mtd4 & mtd5 which would be root1 or 7 & 8 which would be root2

boot_part_p=`grep ubi.mtd=8 /proc/cmdline`

echo "boot part is $boot_part_p "

if [ "$boot_part_p" == "" ] ; then
	echo "not booted from mtd8"
	boot_part_p=`grep ubi.mtd=7 /proc/cmdline`
	if [ "$boot_part_p" == "" ] ; then
 		echo "not booted from mtd7"
		boot_part_p=`grep ubi.mtd=5 /proc/cmdline`
		if [ "$boot_part_p" == "" ] ; then
	 		echo "not booted from mtd5"
			boot_part_p=`grep ubi.mtd=4 /proc/cmdline`
			if [ "$boot_part_p" == "" ] ; then
		 		echo "not booted from mtd4"
		 		echo "Could not determine where rootfs is mounted. Press enter to force an update to rootfs1 or CTRL C to abort"
		 		read
		 		update_flash_root3
			else
				echo "booted from mtd 4"
				update_flash_root2					
			fi
		else
			echo "booted from mtd 5"
			update_flash_root2
		fi
	else
		echo "booted from mtd 7"	
		update_flash_root1
	fi

else
	echo "booted from mtd8"
	update_flash_root1
fi


echo "Bye"

